var AppNotification = (function() {
  // elements
  var $appNotifications = null;
  var $lnkViewNotification = null;
  var $lnkToggleNotification = null;

  var notificationCopy = [];
  var serverOffset = null;
  var currentlyShown;

  var NormalizerService = Services.Normalizer();

  var localStorageKey = 'notificationLastSeen';
  var dropdownHandler = '';

  function init() {
    dropdownHandler = DropdownHandler();
    $appNotifications = jQuery('#appNotifications');
    $lnkViewNotification = jQuery('#lnkViewNotification');
    $lnkToggleNotification = jQuery('#lnkToggleNotification');
    $notificationContainer = $('#appNotificationContainer .dropdown-menu');
    // show default # of notifications
    currentlyShown = dropdownHandler.toggleShown(PBConstants.NOTIFICATIONS.MAX, PBConstants.NOTIFICATIONS);

    // show empty notification list
    dropdownHandler.showMessage(i18next.t('constantMsgs.noNotifications'), $appNotifications, $lnkViewNotification);
    // Add the events
    addEvents();
  }

  function display(notificationList, timezoneOffset) {
    serverOffset = timezoneOffset;

    if (!Array.isArray(notificationList)) {
      notificationList = [];
    }

    // Generate default notification html.
    showNotifications(
      notificationList.slice(0, currentlyShown.count),
      serverOffset
    );
    $lnkViewNotification.text(currentlyShown.text);

    // keep a copy of the first X notifications for "View all"
    notificationCopy = notificationList.slice(0, PBConstants.NOTIFICATIONS.MAX);
  }

  function addEvents() {
    $lnkViewNotification.click(function() {
      currentlyShown = dropdownHandler.toggleShown(currentlyShown.count, PBConstants.NOTIFICATIONS);

      showNotifications(
        notificationCopy.slice(0, currentlyShown.count),
        serverOffset
      );
      $lnkViewNotification.text(currentlyShown.text);

      return false;
    });

    $lnkToggleNotification.click(function(event) {
      event.preventDefault();
      displayUnseenCount(0);
      if (notificationCopy.length > 0) {
        dropdownHandler.setLastSeen(notificationCopy, 'date', localStorageKey);
      }
    });
  }

  function showNotifications(notifications, serverOffset) {
    $appNotifications.empty();
    if (!notifications || notifications.length === 0) {
      // Adding no-notification class when notification length is zero.
      $notificationContainer.addClass('no-notification');
      // no notifications found, show empty and hide the "view all / view less" button
      dropdownHandler.showMessage(i18next.t('constantMsgs.noNotifications'), $appNotifications, $lnkViewNotification);
      return;
    }
    // Removing class no-notification if notification length is not zero.
    if ($notificationContainer.hasClass('no-notification')) {
      $notificationContainer.removeClass('no-notification');
    }

    // notifications found, show the "view all / view less" button.
    $lnkViewNotification.show();

    var lastSeen = dropdownHandler.getLastSeen(localStorageKey);
    var lastSeenDt = new Date(lastSeen);
    var unseenNotification = 0;

    var notificationHtml = '';
    var currentTheme = localStorage.getItem(NormalizerService.generateLocalStorageKey('currentTheme'));
    currentTheme = currentTheme ? currentTheme : PBConstants.DEFAULT_THEME;
    var iconColor = currentTheme + '-theme-color';
    for (var i = 0; i !== notifications.length; ++i) {
      var currNotification = notifications[i];
      var date = new Date(
        DateTimeHelper.getUTCSuffixDate(currNotification.date)
      );
      if (date.getTime() > lastSeenDt.getTime()) {
        ++unseenNotification;
      }
      var time = DateTimeHelper.getDateAndTimeAgoForNotifications(
        currNotification.date,
        serverOffset
      );
      time = time[0] + ' (' + time[1] + ')';
      notificationHtml += getNotificationHtml(
        currNotification.icon,
        currNotification.notification,
        time,
        iconColor
      );
    }

    $appNotifications.append(notificationHtml);

    // update the last seen value.
    displayUnseenCount(unseenNotification);
  }

  function getNotificationHtml(icon, text, time, iconColor) {
    return (
      '<div class="dropdown-item notify-item">' +
      '<div class="notify-icon">' +
      '<i class="mdi ' +
      icon.class +
      '" style="color:' +
      icon[iconColor] +
      '"></i>' +
      '</div>' +
      '<p class="notify-details" title="' +
      text +
      '">' +
      text +
      '<small class="text-muted">' +
      time +
      '</small>' +
      '</p>' +
      '</div>'
    );
  }


  function displayUnseenCount(count) {
    if ($appNotifications.is(':visible')) {
      // notification dropdown is visible nothing to update, so the latest notification
      // must have already been seen ... the last seen.
      dropdownHandler.setLastSeen(notificationCopy, 'date', localStorageKey);
      return;
    }
    $lnkToggleNotification.find('span.badge').hide();
    if (count === 0) {
      return;
    }
    $lnkToggleNotification
      .find('span.badge')
      .html(count)
      .show();
  }

  return {
    display: display,
    init: init
  };
})();
